@@ -35,10 +35,10 @@
 #include <string.h>
 
 #include "pm_c_util.h"
-#include "ppm.h"
+#include "mallocvar.h"
 #include "shhopt.h"
 #include "nstring.h"
-#include "mallocvar.h"
+#include "ppm.h"
 
 #define MAX_LINE (8 * 1024)
   /* The maximum size XPM input line we can handle. */
@@ -55,61 +55,76 @@ const char *xpmColorKeys[] =
  "c",					/* key #5: color visual */
 };
 
-struct cmdline_info {
+struct cmdlineInfo {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    char *input_filespec;  /* Filespecs of input files */
-    char *alpha_filename;
+    const char * input_filespec;  /* Filespecs of input files */
+    const char * alpha_filename;
     int alpha_stdout;
     int verbose;
 };
 
 
-static int verbose;
+static bool verbose;
+
+
+typedef struct {
+    bool none;
+        /* No color is transparent */
+    unsigned int index;
+        /* Color index of the transparent color.
+           Meaningless if 'none'
+        */
+} TransparentColor;
+
 
 
 static void
-parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdline_p) {
+parseCommandLine(int argc, char ** argv,
+                 struct cmdlineInfo *cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
 -----------------------------------------------------------------------------*/
-    optStruct *option_def = malloc(100*sizeof(optStruct));
+    optEntry * option_def;
         /* Instructions to OptParseOptions2 on how to parse our options.
          */
-    optStruct2 opt;
+    optStruct3 opt;
 
     unsigned int option_def_index;
 
-    option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENTRY(0,   "alphaout",   OPT_STRING, &cmdline_p->alpha_filename, 0);
-    OPTENTRY(0,   "verbose",    OPT_FLAG,   &cmdline_p->verbose,        0);
+    MALLOCARRAY_NOFAIL(option_def, 100);
 
-    cmdline_p->alpha_filename = NULL;
-    cmdline_p->verbose = FALSE;
+    option_def_index = 0;   /* incremented by OPTENT3 */
+    OPTENT3(0,   "alphaout",   OPT_STRING, &cmdlineP->alpha_filename,
+            NULL, 0);
+    OPTENT3(0,   "verbose",    OPT_FLAG,   &cmdlineP->verbose,
+            NULL, 0);
+
+    cmdlineP->alpha_filename = NULL;
+    cmdlineP->verbose = FALSE;
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
     opt.allowNegNum = TRUE;  /* We may have parms that are negative numbers */
 
-    optParseOptions2(&argc, argv, opt, 0);
-        /* Uses and sets argc, argv, and some of *cmdline_p and others. */
+    pm_optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
+        /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
     if (argc - 1 == 0)
-        cmdline_p->input_filespec = NULL;  /* he wants stdin */
+        cmdlineP->input_filespec = NULL;  /* he wants stdin */
     else if (argc - 1 == 1)
-        cmdline_p->input_filespec = strdup(argv[1]);
+        cmdlineP->input_filespec = strdup(argv[1]);
     else 
         pm_error("Too many arguments.  The only argument accepted\n"
                  "is the input file specification");
 
-    if (cmdline_p->alpha_filename && 
-        streq(cmdline_p->alpha_filename, "-"))
-        cmdline_p->alpha_stdout = TRUE;
+    if (cmdlineP->alpha_filename && 
+        streq(cmdlineP->alpha_filename, "-"))
+        cmdlineP->alpha_stdout = TRUE;
     else 
-        cmdline_p->alpha_stdout = FALSE;
+        cmdlineP->alpha_stdout = FALSE;
 
 }
 
@@ -162,15 +177,14 @@ getline(char * const line,
 static unsigned int
 getColorNumber(const char * const pArg,
                unsigned int const bytesPerPixel,
-               unsigned int const ncolors) {
+               unsigned int const nColors) {
 /*----------------------------------------------------------------------------
    Return the color number (palette index) at 'p'.
 
    It occupies 'bytesPerPixel' bytes.
 
    Abort program if the number is too large for the format described
-   by 'bytesPerPixel' and 'ncolors'.
-
+   by 'bytesPerPixel' and 'nColors'.
 -----------------------------------------------------------------------------*/
     /* Color number is encoded as pure binary, big-endian, unsigned.
        (which is stupid, since the rest of the file is text)
@@ -185,10 +199,10 @@ getColorNumber(const char * const pArg,
     for (q = p, retval = 0; q < p + bytesPerPixel; ++q)
         retval = (retval << 8) + *q;
 
-    if (bytesPerPixel > 2 && retval >= ncolors)
+    if (bytesPerPixel > 2 && retval >= nColors)
         pm_error("Color number %u in color map is too large, as the "
                  "header says there are only %u colors in the image",
-                 retval, ncolors);
+                 retval, nColors);
 
     return retval;
 }
@@ -213,56 +227,61 @@ getword(char * const output, char ** const cursorP) {
 
 
 static void
-addToColorMap(unsigned int   const seqNum,
-              unsigned int   const colorNumber, 
-              pixel *        const colors,
-              unsigned int * const ptab, 
-              char           const colorspec[],
-              bool           const isTransparent,
-              int *          const transparentP) {
+addToColorMap(unsigned int       const seqNum,
+              unsigned int       const colorNumber, 
+              pixel *            const colors,
+              unsigned int *     const ptab, 
+              char               const colorspec[],
+              bool               const isTransparent,
+              TransparentColor * const transparentP) {
 /*----------------------------------------------------------------------------
    Add the color named by colorspec[] to the colormap contained in
    'colors' and 'ptab', as the color associated with XPM color number
    'colorNumber', which is the seqNum'th color in the XPM color map.
 
-   Iff 'transparent', set *transparentP to the colormap index that 
+   Iff 'isTransparent', set *transparentP to the colormap index that 
    corresponds to this color.
 -----------------------------------------------------------------------------*/
     if (ptab == NULL) {
         /* Index into table. */
         colors[colorNumber] = ppm_parsecolor(colorspec,
                                              (pixval) PPM_MAXMAXVAL);
-        if (isTransparent) 
-            *transparentP = colorNumber;
+        if (isTransparent) {
+            transparentP->none = false;
+            transparentP->index = colorNumber;
+        }
     } else {
         /* Set up linear search table. */
         colors[seqNum] = ppm_parsecolor(colorspec,
                                         (pixval) PPM_MAXMAXVAL);
         ptab[seqNum] = colorNumber;
-        if (isTransparent)
-            *transparentP = seqNum;
+        if (isTransparent) {
+            transparentP->none = false;
+            transparentP->index = seqNum;
+        }
     }
 }
 
 
 
 static void
-interpretXpm3ColorTableLine(char           const line[],
-                            unsigned int   const seqNum, 
-                            unsigned int   const charsPerPixel,
-                            pixel *        const colors,
-                            unsigned int * const ptab,
-                            unsigned int   const ncolors,
-                            int *          const transparentP) {
+interpretXpm3ColorTableLine(char               const line[],
+                            unsigned int       const seqNum, 
+                            unsigned int       const charsPerPixel,
+                            pixel *            const colors,
+                            unsigned int *     const ptab,
+                            unsigned int       const nColors,
+                            TransparentColor * const transparentP) {
 /*----------------------------------------------------------------------------
-   Interpret one line of the color table in the XPM header.  'line' is
-   the line from the XPM file.  It is the seqNum'th color table entry in
-   the file.  The file uses 'charsPerPixel' characters per pixel.
+   Interpret one line of the color table in the XPM header.  'line' is the
+   line from the XPM file.  It is the seqNum'th color table entry in the file.
+   The raster in the file uses 'charsPerPixel' characters per pixel (i.e.
+   a color number (palette index) is 'charsPerPixel' characters).
 
-   Add the information from this color table entry to the color table
-   'colors' and, if it isn't NULL, the corresponding lookup shadow table
-   'ptab' (see readXpm3ColorTable for a description of these data 
-   structures).
+   Add the information from this color table entry to the color table 'colors'
+   and, if it isn't NULL, the corresponding lookup shadow table 'ptab'.  Both
+   are of size 'nColors'.  (See readV3ColorTable for a description of these
+   data structures).
 
    The line may include values for multiple kinds of color (grayscale,
    color, etc.).  We take the highest of these (e.g. color over grayscale).
@@ -272,12 +291,12 @@ interpretXpm3ColorTableLine(char           const line[],
 -----------------------------------------------------------------------------*/
     /* Note: this code seems to allow for multi-word color specifications,
        but I'm not aware that such are legal.  Ultimately, ppm_parsecolor()
-       interprets the name, and I believe it only takes single word 
+       interprets the name, and I believe it takes only single word 
        color specifications.  -Bryan 2001.05.06.
     */
     char str2[MAX_LINE+1];    
-    char *t1;
-    char *t2;
+    char * t1;
+    char * t2;
     int endOfEntry;   /* boolean */
     
     unsigned int curkey, key, highkey;	/* current color key */
@@ -298,9 +317,9 @@ interpretXpm3ColorTableLine(char           const line[],
                  "It is the %uth entry in the color table.", 
                  line, seqNum);
     else
-        t1++;  /* Points now to first color number character */
+        ++t1;  /* Points now to first color number character */
 
-    colorNumber = getColorNumber(t1, charsPerPixel, ncolors);
+    colorNumber = getColorNumber(t1, charsPerPixel, nColors);
     t1 += charsPerPixel;
 
     /*
@@ -334,8 +353,8 @@ interpretXpm3ColorTableLine(char           const line[],
                              "'%s' before '%s'.", line, str2);
                 if (!lastwaskey) 
                     strcat(curbuf, " ");		/* append space */
-                if ( (strncmp(str2, "None", 4) == 0) 
-                     || (strncmp(str2, "none", 4) == 0) ) {
+                if ( (strneq(str2, "None", 4)) 
+                     || (strneq(str2, "none", 4)) ) {
                     /* This entry identifies the transparent color number */
                     strcat(curbuf, "#000000");  /* Make it black */
                     isTransparent = TRUE;
@@ -377,16 +396,72 @@ interpretXpm3ColorTableLine(char           const line[],
 
 
 static void
-readXpm3Header(FILE *          const stream,
-               int *           const widthP,
-               int *           const heightP, 
-               unsigned int *  const charsPerPixelP,
-               int *           const nColorsP,
-               pixel **        const colorsP,
-               unsigned int ** const ptabP,
-               int *           const transparentP) {
+readV3ColorTable(FILE *             const ifP,
+                 pixel **           const colorsP,
+                 unsigned int       const nColors,
+                 unsigned int       const charsPerPixel,
+                 unsigned int **    const ptabP,
+                 TransparentColor * const transparentP) {
 /*----------------------------------------------------------------------------
-  Read the header of the XPM file on stream 'stream'.  Assume the
+   Read the color table from the XPM Version 3 header.
+
+   Assume *ifP is positioned to the color table; leave it positioned after.
+-----------------------------------------------------------------------------*/
+    unsigned int colormapSize;
+    pixel * colors;  /* malloc'ed array */
+    unsigned int * ptab; /* malloc'ed array */
+
+    if (charsPerPixel <= 2) {
+        /* Set up direct index (see above) */
+        colormapSize =
+            charsPerPixel == 0 ?   1 :
+            charsPerPixel == 1 ? 256 :
+            256*256;
+        ptab = NULL;
+    } else {
+        /* Set up lookup table (see above) */
+        colormapSize = nColors;
+        MALLOCARRAY(ptab, nColors);
+        if (ptab == NULL)
+            pm_error("Unable to allocate memory for %u colors", nColors);
+    }
+    colors = ppm_allocrow(colormapSize);
+    
+    { 
+        /* Read the color table */
+        unsigned int seqNum;
+            /* Sequence number of entry within color table in XPM header */
+
+        transparentP->none = true;  /* initial value */
+
+        for (seqNum = 0; seqNum < nColors; ++seqNum) {
+            char line[MAX_LINE+1];
+            getline(line, sizeof(line), ifP);
+            /* skip the comment line if any */
+            if (strneq(line, "/*", 2))
+                getline(line, sizeof(line), ifP);
+            
+            interpretXpm3ColorTableLine(line, seqNum, charsPerPixel,
+                                        colors, ptab, nColors, transparentP);
+        }
+    }
+    *colorsP = colors;
+    *ptabP   = ptab;
+}
+
+
+
+static void
+readXpm3Header(FILE *             const ifP,
+               unsigned int *     const widthP,
+               unsigned int *     const heightP, 
+               unsigned int *     const charsPerPixelP,
+               unsigned int *     const nColorsP,
+               pixel **           const colorsP,
+               unsigned int **    const ptabP,
+               TransparentColor * const transparentP) {
+/*----------------------------------------------------------------------------
+  Read the header of the XPM file on stream *ifP.  Assume the
   getline() stream is presently positioned to the beginning of the
   file and it is a Version 3 XPM file.  Leave the stream positioned
   after the header.
@@ -415,120 +490,168 @@ readXpm3Header(FILE *          const stream,
     char line[MAX_LINE+1];
     const char * xpm3_signature = "/* XPM */";
     
-    unsigned int colormapSize;
     unsigned int width, height;
     unsigned int nColors;
     unsigned int charsPerPixel;
-    pixel * colors;
-    unsigned int * ptab;
 
     /* Read the XPM signature comment */
-    getline(line, sizeof(line), stream);
-    if (strncmp(line, xpm3_signature, strlen(xpm3_signature)) != 0) 
+    getline(line, sizeof(line), ifP);
+    if (!strneq(line, xpm3_signature, strlen(xpm3_signature))) 
         pm_error("Apparent XPM 3 file does not start with '/* XPM */'.  "
                  "First line is '%s'", xpm3_signature);
 
     /* Read the assignment line */
-    getline(line, sizeof(line), stream);
-    if (strncmp(line, "static char", 11) != 0)
+    getline(line, sizeof(line), ifP);
+    if (!strneq(line, "static char", 11))
         pm_error("Cannot find data structure declaration.  Expected a "
                  "line starting with 'static char', but found the line "
                  "'%s'.", line);
 
-	/* Read the hints line */
-    getline(line, sizeof(line), stream);
-    /* skip the comment line if any */
-    if (!strncmp(line, "/*", 2)) {
+    getline(line, sizeof(line), ifP);
+
+    /* Skip the comment block, if one starts here */
+    if (strneq(line, "/*", 2)) {
         while (!strstr(line, "*/"))
-            getline(line, sizeof(line), stream);
-        getline(line, sizeof(line), stream);
+            getline(line, sizeof(line), ifP);
+        getline(line, sizeof(line), ifP);
     }
+
+    /* Parse the hints line */
     if (sscanf(line, "\"%u %u %u %u\",", &width, &height,
                &nColors, &charsPerPixel) != 4)
         pm_error("error scanning hints line");
 
-    if (verbose == 1) {
+    if (verbose) {
         pm_message("Width x Height:  %u x %u", width, height);
         pm_message("no. of colors:  %u", nColors);
         pm_message("chars per pixel:  %u", charsPerPixel);
     }
 
-    /* Allocate space for color table. */
-    if (charsPerPixel <= 2) {
-        /* Set up direct index (see above) */
-        colormapSize = charsPerPixel == 1 ? 256 : 256*256;
-        ptab = NULL;
-    } else {
-        /* Set up lookup table (see above) */
-        colormapSize = nColors;
-        MALLOCARRAY(ptab, nColors);
-        if (ptab == NULL)
-            pm_error("Unable to allocate memory for %u colors", nColors);
-    }
-    colors = ppm_allocrow(colormapSize);
-    
-    { 
-        /* Read the color table */
-        unsigned int seqNum;
-            /* Sequence number of entry within color table in XPM header */
+    readV3ColorTable(ifP, colorsP, nColors, charsPerPixel, ptabP,
+                     transparentP);
 
-        *transparentP = -1;  /* initial value */
-
-        for (seqNum = 0; seqNum < nColors; ++seqNum) {
-            getline(line, sizeof(line), stream);
-            /* skip the comment line if any */
-            if (!strncmp(line, "/*", 2))
-                getline(line, sizeof(line), stream);
-            
-            interpretXpm3ColorTableLine(line, seqNum, charsPerPixel,
-                                        colors, ptab, nColors, transparentP);
-        }
-    }
     *widthP         = width;
     *heightP        = height;
     *charsPerPixelP = charsPerPixel;
     *nColorsP       = nColors;
-    *colorsP        = colors;
-    *ptabP          = ptab;
 }
 
 
 
 static void
-readXpm1Header(FILE *          const stream,
-               int *           const widthP,
-               int *           const heightP, 
+readV1ColorTable(FILE *          const ifP,
+                 pixel **        const colorsP,
+                 unsigned int    const nColors,
+                 unsigned int    const charsPerPixel,
+                 unsigned int ** const ptabP) {
+/*----------------------------------------------------------------------------
+   Read the color table from the XPM Version 1 header.
+
+   Assume *ifP is positioned to the color table; leave it positioned after.
+-----------------------------------------------------------------------------*/
+    pixel * colors;  /* malloc'ed array */
+    unsigned int * ptab; /* malloc'ed array */
+    unsigned int i;
+
+    /* Allocate space for color table. */
+    if (charsPerPixel <= 2) {
+        /* Up to two chars per pixel, we can use an indexed table. */
+        unsigned int v;
+        v = 1;
+        for (i = 0; i < charsPerPixel; ++i)
+            v *= 256;
+        colors = ppm_allocrow(v);
+        ptab = NULL;
+    } else {
+        /* Over two chars per pixel, we fall back on linear search. */
+        colors = ppm_allocrow(nColors);
+        MALLOCARRAY(ptab, nColors);
+        if (ptab == NULL)
+            pm_error("Unable to allocate memory for %u colors", nColors);
+    }
+
+    for (i = 0; i < nColors; ++i) {
+        char line[MAX_LINE+1];
+        char str1[MAX_LINE+1];
+        char str2[MAX_LINE+1];
+        char * t1;
+        char * t2;
+
+        getline(line, sizeof(line), ifP);
+
+        if ((t1 = strchr(line, '"')) == NULL)
+            pm_error("D error scanning color table");
+        if ((t2 = strchr(t1 + 1, '"')) == NULL)
+            pm_error("E error scanning color table");
+        if (t2 - t1 - 1 != charsPerPixel)
+            pm_error("wrong number of chars per pixel in color table");
+        strncpy(str1, t1 + 1, t2 - t1 - 1);
+        str1[t2 - t1 - 1] = '\0';
+
+        if ((t1 = strchr(t2 + 1, '"')) == NULL)
+            pm_error("F error scanning color table");
+        if ((t2 = strchr(t1 + 1, '"')) == NULL)
+            pm_error("G error scanning color table");
+        strncpy(str2, t1 + 1, t2 - t1 - 1);
+        str2[t2 - t1 - 1] = '\0';
+
+        {
+            unsigned int v;
+            unsigned int j;
+
+            v = 0;
+            for (j = 0; j < charsPerPixel; ++j)
+                v = (v << 8) + str1[j];
+            if (charsPerPixel <= 2)
+                /* Index into table. */
+                colors[v] = ppm_parsecolor(str2, PPM_MAXMAXVAL);
+            else {
+                /* Set up linear search table. */
+                colors[i] = ppm_parsecolor(str2, PPM_MAXMAXVAL);
+                ptab[i] = v;
+            }
+        }
+    }
+    *colorsP = colors;
+    *ptabP   = ptab;
+}
+
+
+
+static void
+readXpm1Header(FILE *          const ifP,
+               unsigned int *  const widthP,
+               unsigned int *  const heightP, 
                unsigned int *  const charsPerPixelP,
-               int *           const ncolorsP, 
+               unsigned int *  const nColorsP, 
                pixel **        const colorsP,
                unsigned int ** const ptabP) {
 /*----------------------------------------------------------------------------
-  Read the header of the XPM file on stream 'stream'.  Assume the
+  Read the header of the XPM file on stream *ifP.  Assume the
   getline() stream is presently positioned to the beginning of the
   file and it is a Version 1 XPM file.  Leave the stream positioned
   after the header.
   
   Return the information from the header the same as for readXpm3Header.
 -----------------------------------------------------------------------------*/
-    char line[MAX_LINE+1], str1[MAX_LINE+1], str2[MAX_LINE+1];
-    char *t1;
-    char *t2;
     int format, v;
-    int i, j;
     bool processedStaticChar;  
         /* We have read up to and interpreted the "static char..." line */
     bool gotPixel;
+    char * t1;
 
-    *widthP = *heightP = *ncolorsP = format = -1;
+    *widthP = *heightP = *nColorsP = format = -1;
     gotPixel = false;
 
     /* Read the initial defines. */
     processedStaticChar = FALSE;
     while (!processedStaticChar) {
-        getline(line, sizeof(line), stream);
+        char line[MAX_LINE+1];
+        char str1[MAX_LINE+1];
+
+        getline(line, sizeof(line), ifP);
 
         if (sscanf(line, "#define %s %d", str1, &v) == 2) {
-            char *t1;
             if ((t1 = strrchr(str1, '_')) == NULL)
                 t1 = str1;
             else
@@ -539,13 +662,13 @@ readXpm1Header(FILE *          const stream,
                 *widthP = v;
             else if (streq(t1, "height"))
                 *heightP = v;
-            else if (streq(t1, "ncolors"))
-                *ncolorsP = v;
+            else if (streq(t1, "nColors"))
+                *nColorsP = v;
             else if (streq(t1, "pixel")) {
                 gotPixel = TRUE;
                 *charsPerPixelP = v;
             }
-        } else if (!strncmp(line, "static char", 11)) {
+        } else if (strneq(line, "static char", 11)) {
             if ((t1 = strrchr(line, '_')) == NULL)
                 t1 = line;
             else
@@ -567,76 +690,30 @@ readXpm1Header(FILE *          const stream,
         pm_error("missing or invalid width");
     if (*heightP == -1)
         pm_error("missing or invalid height");
-    if (*ncolorsP == -1)
-        pm_error("missing or invalid ncolors");
+    if (*nColorsP == -1)
+        pm_error("missing or invalid nColors");
 
     if (*charsPerPixelP > 2)
         pm_message("WARNING: > 2 characters per pixel uses a lot of memory");
 
     /* If there's a monochrome color table, skip it. */
-    if (!strncmp(t1, "mono", 4)) {
+    if (strneq(t1, "mono", 4)) {
         for (;;) {
-            getline(line, sizeof(line), stream);
-            if (!strncmp(line, "static char", 11))
+            char line[MAX_LINE+1];
+            getline(line, sizeof(line), ifP);
+            if (strneq(line, "static char", 11))
                 break;
         }
     }
-    /* Allocate space for color table. */
-    if (*charsPerPixelP <= 2) {
-        /* Up to two chars per pixel, we can use an indexed table. */
-        v = 1;
-        for (i = 0; i < *charsPerPixelP; ++i)
-            v *= 256;
-        *colorsP = ppm_allocrow(v);
-        *ptabP = NULL;
-    } else {
-        /* Over two chars per pixel, we fall back on linear search. */
-        *colorsP = ppm_allocrow(*ncolorsP);
-        MALLOCARRAY(*ptabP, *ncolorsP);
-        if (*ptabP == NULL)
-            pm_error("Unable to allocate memory for %d colors", *ncolorsP);
-    }
+    readV1ColorTable(ifP, colorsP, *nColorsP, *charsPerPixelP, ptabP);
 
-    /* Read color table. */
-    for (i = 0; i < *ncolorsP; ++i) {
-        getline(line, sizeof(line), stream);
-
-        if ((t1 = strchr(line, '"')) == NULL)
-            pm_error("D error scanning color table");
-        if ((t2 = strchr(t1 + 1, '"')) == NULL)
-            pm_error("E error scanning color table");
-        if (t2 - t1 - 1 != *charsPerPixelP)
-            pm_error("wrong number of chars per pixel in color table");
-        strncpy(str1, t1 + 1, t2 - t1 - 1);
-        str1[t2 - t1 - 1] = '\0';
-
-        if ((t1 = strchr(t2 + 1, '"')) == NULL)
-            pm_error("F error scanning color table");
-        if ((t2 = strchr(t1 + 1, '"')) == NULL)
-            pm_error("G error scanning color table");
-        strncpy(str2, t1 + 1, t2 - t1 - 1);
-        str2[t2 - t1 - 1] = '\0';
-
-        v = 0;
-        for (j = 0; j < *charsPerPixelP; ++j)
-            v = (v << 8) + str1[j];
-        if (*charsPerPixelP <= 2)
-            /* Index into table. */
-            (*colorsP)[v] = ppm_parsecolor(str2,
-                                           (pixval) PPM_MAXMAXVAL);
-        else {
-            /* Set up linear search table. */
-            (*colorsP)[i] = ppm_parsecolor(str2,
-                                           (pixval) PPM_MAXMAXVAL);
-            (*ptabP)[i] = v;
-        }
-    }
     /* Position to first line of raster (which is the line after
        "static char ...").
     */
     for (;;) {
-        getline(line, sizeof(line), stream);
-        if (strncmp(line, "static char", 11) == 0)
+        char line[MAX_LINE+1];
+        getline(line, sizeof(line), ifP);
+        if (strneq(line, "static char", 11))
             break;
     }
 }
@@ -644,17 +721,17 @@ readXpm1Header(FILE *          const stream,
 
 
 static void
-interpretXpmLine(char           const line[],
-                 unsigned int   const charsPerPixel,
-                 int            const ncolors,
-                 unsigned int * const ptab, 
-                 int **         const cursorP,
-                 int *          const maxCursor) {
+interpretXpmLine(char            const line[],
+                 unsigned int    const width,
+                 unsigned int    const charsPerPixel,
+                 unsigned int    const nColors,
+                 unsigned int *  const ptab, 
+                 unsigned int ** const cursorP) {
 /*----------------------------------------------------------------------------
-   Interpret one line of XPM input.  The line is in 'line', and its
-   format is 'charsPerPixel' characters per pixel.  'ptab' is the
-   color table that applies to the line, which table has 'ncolors'
-   colors.
+   Interpret one line from XPM input which describes one raster line of the
+   image.  The XPM line is in 'line', and its format is 'width' pixel,
+   'charsPerPixel' characters per pixel.  'ptab' is the color table that
+   applies to the line, which table has 'nColors' colors.
 
    Put the colormap indexes for the pixels represented in 'line' at
    *cursorP, lined up in the order they are in 'line', and return
@@ -663,9 +740,9 @@ interpretXpmLine(char           const line[],
    If the line doesn't start with a quote (e.g. it is empty), we issue
    a warning and just treat the line as one that describes no pixels.
 
-   Stop processing the line either at the end of the line or when
-   the output would go beyond maxCursor, whichever comes first.
+   Abort program if there aren't exactly 'width' pixels in the line.
 -----------------------------------------------------------------------------*/
+    unsigned int pixelCtSoFar;
     char * lineCursor;
 
     lineCursor = strchr(line, '"');  /* position to 1st quote in line */
@@ -682,13 +759,23 @@ interpretXpmLine(char           const line[],
         /* Handle pixels until a close quote, eol, or we've returned all
            the pixels Caller wants.
         */
-        while (*lineCursor && *lineCursor != '"' && *cursorP <= maxCursor) {
+        for (pixelCtSoFar = 0; pixelCtSoFar < width; ++pixelCtSoFar) {
             unsigned int colorNumber;
             unsigned int i;
             colorNumber = 0;  /* initial value */
             for (i = 0; i < charsPerPixel; ++i) {
-                unsigned char const byte = (unsigned char)(*lineCursor++);
-                colorNumber = (colorNumber << 8) + byte;
+                if (*lineCursor == '\0')
+                    pm_error("XPM input file ends in the middle of a string "
+                             "that represents a raster line");
+                if (*lineCursor == '"')
+                    pm_error("A string that represents a raster line in the "
+                             "XPM input file is too short to contain "
+                             "%u pixels (%u characters each)",
+                             width, charsPerPixel);
+                {
+                    unsigned char const byte = (unsigned char)(*lineCursor++);
+                    colorNumber = (colorNumber << 8) + byte;
+                }
             }
             if (ptab == NULL)
                 /* colormap is indexed directly by XPM color number */
@@ -696,8 +783,8 @@ interpretXpmLine(char           const line[],
             else {
                 /* colormap shadows ptab[].  Find this color # in ptab[] */
                 unsigned int i;
-                for (i = 0; i < ncolors && ptab[i] != colorNumber; ++i);
-                if (i < ncolors)
+                for (i = 0; i < nColors && ptab[i] != colorNumber; ++i);
+                if (i < nColors)
                     *(*cursorP)++ = i;
                 else
                     pm_error("Color number %u is in raster, but not in "
@@ -705,85 +792,99 @@ interpretXpmLine(char           const line[],
                              colorNumber, line);
             }
         }
+        if (*lineCursor != '"')
+            pm_error("A raster line continues past width of image");
     }
 }
 
 
 
 static void
-ReadXPMFile(FILE * const stream, int * const widthP, int * const heightP, 
-            pixel ** const colorsP, int ** const dataP, 
-            int * const transparentP) {
+readXpmFile(FILE *             const ifP,
+            unsigned int *     const widthP,
+            unsigned int *     const heightP, 
+            pixel **           const colorsP,
+            unsigned int **    const dataP, 
+            TransparentColor * const transparentP) {
 /*----------------------------------------------------------------------------
-   Read the XPM file from stream 'stream'.
+   Read the XPM file from stream *ifP.
 
    Return the dimensions of the image as *widthP and *heightP.
-   Return the color map as *colorsP, which is an array of *ncolorsP
+   Return the color map as *colorsP, which is an array of *nColorsP
    colors.
 
    Return the raster in newly malloced storage, an array of *widthP by
    *heightP integers, each of which is an index into the colormap
-   *colorsP (and therefore less than *ncolorsP).  Return the address
+   *colorsP (and therefore less than *nColorsP).  Return the address
    of the array as *dataP.
 
    In the colormap, put black for the transparent color, if the XPM 
    image contains one.
 -----------------------------------------------------------------------------*/
+    unsigned int * data;
     char line[MAX_LINE+1], str1[MAX_LINE+1];
-    int totalpixels;
-    int * cursor;  /* cursor into *dataP */
-    int * maxcursor;  /* value of above cursor for last pixel in image */
+    unsigned int totalpixels;
+    unsigned int * cursor;
+        /* cursor into data{} */
+    unsigned int * maxCursor;
+        /* value of above cursor for last pixel in image */
     unsigned int * ptab;   /* colormap - malloc'ed */
     int rc;
-    int ncolors;
+    unsigned int nColors;
     unsigned int charsPerPixel;
+    unsigned int width, height;
 
     backup = FALSE;
 
     /* Read the header line */
-    getline(line, sizeof(line), stream);
+    getline(line, sizeof(line), ifP);
     backup = TRUE;  /* back up so next read reads this line again */
     
     rc = sscanf(line, "/* %s */", str1);
-    if (rc == 1 && strncmp(str1, "XPM", 3) == 0) {
+    if (rc == 1 && strneq(str1, "XPM", 3)) {
         /* It's an XPM version 3 file */
-        readXpm3Header(stream, widthP, heightP, &charsPerPixel,
-                       &ncolors, colorsP, &ptab, transparentP);
+        readXpm3Header(ifP, &width, &height, &charsPerPixel,
+                       &nColors, colorsP, &ptab, transparentP);
     } else {				/* try as an XPM version 1 file */
         /* Assume it's an XPM version 1 file */
-        readXpm1Header(stream, widthP, heightP, &charsPerPixel, 
-                       &ncolors, colorsP, &ptab);
-        *transparentP = -1;  /* No transparency in version 1 */
+        readXpm1Header(ifP, &width, &height, &charsPerPixel, 
+                       &nColors, colorsP, &ptab);
+        transparentP->none = true;  /* No transparency in version 1 */
     }
-    totalpixels = *widthP * *heightP;
-    MALLOCARRAY(*dataP, totalpixels);
-    if (*dataP == NULL)
-        pm_error("Could not get %d bytes of memory for image", totalpixels);
-    cursor = *dataP;
-    maxcursor = *dataP + totalpixels - 1;
-	getline(line, sizeof(line), stream); 
+    totalpixels = width * height;
+    MALLOCARRAY(data, totalpixels);
+    if (!data)
+        pm_error("Could not get %u bytes of memory for image", totalpixels);
+    cursor = &data[0];
+    maxCursor = &data[totalpixels - 1];
+	getline(line, sizeof(line), ifP); 
         /* read next line (first line may not always start with comment) */
-    while (cursor <= maxcursor) {
-        if (strncmp(line, "/*", 2) == 0) {
+    while (cursor <= maxCursor) {
+        if (strneq(line, "/*", 2)) {
             /* It's a comment.  Ignore it. */
         } else {
-            interpretXpmLine(line, charsPerPixel, 
-                             ncolors, ptab, &cursor, maxcursor);
+            interpretXpmLine(line, width, charsPerPixel, nColors, ptab,
+                             &cursor);
         }
-        if (cursor <= maxcursor)
-            getline(line, sizeof(line), stream);
+        if (cursor <= maxCursor)
+            getline(line, sizeof(line), ifP);
     }
     if (ptab) free(ptab);
+    *dataP   = data;
+    *widthP  = width;
+    *heightP = height;
 }
  
 
 
 static void
-writeOutput(FILE * const imageout_file,
-            FILE * const alpha_file,
-            int const cols, int const rows, 
-            pixel * const colors, int * const data,
-            int transparent) {
+writeOutput(FILE *           const imageout_file,
+            FILE *           const alpha_file,
+            unsigned int     const cols,
+            unsigned int     const rows, 
+            pixel *          const colors,
+            unsigned int *   const data,
+            TransparentColor const transparent) {
 /*----------------------------------------------------------------------------
    Write the image in 'data' to open PPM file stream 'imageout_file',
    and the alpha mask for it to open PBM file stream 'alpha_file',
@@ -795,8 +896,8 @@ writeOutput(FILE * const imageout_file,
    Where the index 'transparent' occurs in 'data', the pixel is supposed
    to be transparent.  If 'transparent' < 0, no pixels are transparent.
 -----------------------------------------------------------------------------*/
-    int row;
-    pixel *pixrow;
+    unsigned int row;
+    pixel * pixrow;
     bit * alpharow;
 
     if (imageout_file)
@@ -808,12 +909,13 @@ writeOutput(FILE * const imageout_file,
     alpharow = pbm_allocrow(cols);
 
     for (row = 0; row < rows; ++row ) {
-        int col;
-        int * const datarow = data+(row*cols);
+        unsigned int * const datarow = data+(row*cols);
+
+        unsigned int col;
 
         for (col = 0; col < cols; ++col) {
             pixrow[col] = colors[datarow[col]];
-            if (datarow[col] == transparent)
+            if (!transparent.none && datarow[col] == transparent.index)
                 alpharow[col] = PBM_BLACK;
             else
                 alpharow[col] = PBM_WHITE;
@@ -838,28 +940,30 @@ writeOutput(FILE * const imageout_file,
 int
 main(int argc, char *argv[]) {
 
-    FILE *ifp;
-    FILE *alpha_file, *imageout_file;
-    pixel *colormap;
-    int cols, rows;
-    int transparent;  /* value of 'data' that means transparent */
-    int *data;
+    FILE * ifP;
+    FILE * alpha_file;
+    FILE * imageout_file;
+    pixel * colormap;
+    unsigned int cols, rows;
+    TransparentColor transparent;
+        /* Pixels of what color, if any, are transparent */
+    unsigned int * data;
         /* The image as an array of width * height integers, each one
            being an index int colormap[].
         */
 
-    struct cmdline_info cmdline;
+    struct cmdlineInfo cmdline;
 
     ppm_init(&argc, argv);
 
-    parse_command_line(argc, argv, &cmdline);
+    parseCommandLine(argc, argv, &cmdline);
 
     verbose = cmdline.verbose;
 
     if ( cmdline.input_filespec != NULL ) 
-        ifp = pm_openr( cmdline.input_filespec);
+        ifP = pm_openr( cmdline.input_filespec);
     else
-        ifp = stdin;
+        ifP = stdin;
 
     if (cmdline.alpha_stdout)
         alpha_file = stdout;
@@ -874,9 +978,9 @@ main(int argc, char *argv[]) {
     else
         imageout_file = stdout;
 
-    ReadXPMFile(ifp, &cols, &rows, &colormap, &data, &transparent);
+    readXpmFile(ifP, &cols, &rows, &colormap, &data, &transparent);
     
-    pm_close(ifp);
+    pm_close(ifP);
 
     writeOutput(imageout_file, alpha_file, cols, rows, colormap, data,
                 transparent);
@@ -885,3 +989,6 @@ main(int argc, char *argv[]) {
     
     return 0;
 }
+
+
+
