@@ -88,7 +88,7 @@ countMatrixOptColumns(const char *   const rowString,
         if (error) {
             pm_error("Unable to parse -matrix value row '%s'.  %s",
                      rowString, error);
-            strfree(error);
+            pm_strfree(error);
         } else {
             ++colCt;
 
@@ -97,7 +97,7 @@ countMatrixOptColumns(const char *   const rowString,
                 assert(*cursor == ',');
                 ++cursor;  /* advance over comma to next column */
             }
-            strfree(colString);
+            pm_strfree(colString);
         }
     }
     *colCtP = colCt;
@@ -133,7 +133,7 @@ getMatrixOptDimensions(const char *   const matrixOptString,
         if (error) {
             pm_error("Unable to parse -matrix value '%s'.  %s",
                      matrixOptString, error);
-            strfree(error);
+            pm_strfree(error);
         } else {
             unsigned int colCt;
             ++rowCt;
@@ -147,7 +147,7 @@ getMatrixOptDimensions(const char *   const matrixOptString,
                 pm_error("-matrix option value contains rows of different "
                          "widths: %u and %u", *widthP, colCt);
             }            
-            strfree(rowString);
+            pm_strfree(rowString);
             cursor = next;
 
             if (*cursor) {
@@ -178,7 +178,7 @@ parseMatrixRow(const char * const matrixOptRowString,
 
         if (error) {
             pm_error("Failed parsing a row in the -matrix value.  %s", error);
-            strfree(error);
+            pm_strfree(error);
         } else {
             if (colString[0] == '\0')
                 pm_error("The Column %u element of the row '%s' in the "
@@ -193,7 +193,7 @@ parseMatrixRow(const char * const matrixOptRowString,
                              "-matrix value is not a valid floating point "
                              "number", col, matrixOptRowString);
             }
-            strfree(colString);
+            pm_strfree(colString);
 
             cursor = next;
 
@@ -225,11 +225,11 @@ parseMatrixOptWithDimensions(const char * const matrixOptString,
 
         if (error) {
             pm_error("Failed parsing -matrix value.  %s", error);
-            strfree(error);
+            pm_strfree(error);
         } else {
             parseMatrixRow(rowString, width, weight[row]);
 
-            strfree(rowString);
+            pm_strfree(rowString);
 
             cursor = next;
 
@@ -297,7 +297,7 @@ parseCommandLine(int argc, char ** argv,
    was passed to us as the argv array.  We also trash *argv.
 -----------------------------------------------------------------------------*/
     optEntry *option_def;
-        /* Instructions to optParseOptions3 on how to parse our options.
+        /* Instructions to pm_optParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
 
@@ -321,7 +321,7 @@ parseCommandLine(int argc, char ** argv,
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
     opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
 
-    optParseOptions3( &argc, argv, opt, sizeof(opt), 0);
+    pm_optParseOptions3( &argc, argv, opt, sizeof(opt), 0);
         /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
     if (matrixfileSpec && cmdlineP->matrixSpec)
@@ -455,13 +455,13 @@ static void
 convKernelCreatePnm(struct pam *         const cpamP,
                     tuple * const *      const ctuples, 
                     unsigned int         const depth,
-                    bool                 const offsetPgm,
+                    bool                 const offsetPnm,
                     struct convKernel ** const convKernelPP) {
 /*----------------------------------------------------------------------------
-   Compute the convolution matrix in normalized form from the PGM
-   form.  Each element of the output matrix is the actual weight we give an
-   input pixel -- i.e. the thing by which we multiple a value from the
-   input image.
+   Compute the convolution matrix in normalized form from the PGM form
+   'ctuples'/'cpamP'.  Each element of the output matrix is the actual weight
+   we give an input pixel -- i.e. the thing by which we multiple a value from
+   the input image.
 
    'depth' is the required number of planes in the kernel.  If 'ctuples' has
    fewer planes than that, we duplicate as necessary.  E.g. if 'ctuples' is
@@ -470,13 +470,13 @@ convKernelCreatePnm(struct pam *         const cpamP,
    'ctuples' has more planes than specified, we ignore the higher numbered
    ones.
 
-   'offsetPgm' means the PGM convolution matrix is defined in offset form so
+   'offsetPnm' means the PNM convolution matrix is defined in offset form so
    that it can represent negative values.  E.g. with maxval 100, 50 means
    0, 100 means 50, and 0 means -50.  If 'offsetPgm' is false, 0 means 0
    and there are no negative weights.
 -----------------------------------------------------------------------------*/
-    double const scale = (offsetPgm ? 2.0 : 1.0) / cpamP->maxval;
-    double const offset = offsetPgm ? - 1.0 : 0.0;
+    double const scale = (offsetPnm ? 2.0 : 1.0) / cpamP->maxval;
+    double const offset = offsetPnm ? - 1.0 : 0.0;
     unsigned int const planes = MIN(3, depth);
 
     struct convKernel * convKernelP;
@@ -579,9 +579,19 @@ normalizeKernel(struct convKernel * const convKernelP) {
 static void
 getKernelPnm(const char *         const fileName,
              unsigned int         const depth,
-             bool                 const nooffset,
+             bool                 const offset,
              struct convKernel ** const convKernelPP) {
+/*----------------------------------------------------------------------------
+   Get the convolution kernel from the PNM file named 'fileName'.
+   'offset' means the PNM convolution matrix is defined in offset form so
+   that it can represent negative values.  E.g. with maxval 100, 50 means
+   0, 100 means 50, and 0 means -50.  If 'offsetPgm' is false, 0 means 0
+   and there are no negative weights.
 
+   Make the kernel suitable for convolving an image of depth 'depth'.
+
+   Return the kernel as *convKernelPP.
+-----------------------------------------------------------------------------*/
     struct pam cpam;
     FILE * cifP;
     tuple ** ctuples;
@@ -594,7 +604,7 @@ getKernelPnm(const char *         const fileName,
     
     validateKernelDimensions(cpam.width, cpam.height);
 
-    convKernelCreatePnm(&cpam, ctuples, depth, nooffset, convKernelPP);
+    convKernelCreatePnm(&cpam, ctuples, depth, offset, convKernelPP);
 }
 
 
@@ -691,7 +701,7 @@ parsePlaneFileLine(const char *   const line,
 
                 ++colCt;
         }
-        strfree(token);
+        pm_strfree(token);
     }
     *weightP = weight;
     *widthP = colCt;
@@ -758,7 +768,7 @@ readPlaneFile(FILE *         const ifP,
                     }                    
                     ++rowCt;
                 }
-                strfree(line);
+                pm_strfree(line);
             }
         }
     }
@@ -889,11 +899,11 @@ getKernel(struct cmdlineInfo   const cmdline,
 
    We generate a kernel object in standard form (free of any indication of
    where it came from) and return a handle to it as *convKernelPP.
------------------------------------------------------------------------------*/
+----------------------------------------------------------------------------*/
     struct convKernel * convKernelP;
 
     if (cmdline.pnmMatrixFileName)
-        getKernelPnm(cmdline.pnmMatrixFileName, depth, cmdline.nooffset,
+        getKernelPnm(cmdline.pnmMatrixFileName, depth, !cmdline.nooffset,
                      &convKernelP);
     else if (cmdline.matrixfile)
         convKernelCreateSimpleFile(cmdline.matrixfile, cmdline.normalize,
