@@ -8,6 +8,7 @@
   Netpbm library subroutines.
 **************************************************************************/
 
+#define _BSD_SOURCE          /* Make sure strdup is in string.h */
 #define _XOPEN_SOURCE 500    /* Make sure ftello, fseeko are defined */
 
 #include "netpbm/pm_config.h"
@@ -115,8 +116,8 @@ pm_fork(int *         const iAmParentP,
     rc = fork();
 
     if (rc < 0) {
-        asprintfN(errorP, "Failed to fork a process.  errno=%d (%s)",
-                  errno, strerror(errno));
+        pm_asprintf(errorP, "Failed to fork a process.  errno=%d (%s)",
+                    errno, strerror(errno));
     } else {
         *errorP = NULL;
 
@@ -128,8 +129,8 @@ pm_fork(int *         const iAmParentP,
         }
     }
 #else
-    asprintfN(errorP, "Cannot fork a process, because this system does "
-              "not have POSIX fork()");
+    pm_asprintf(errorP, "Cannot fork a process, because this system does "
+                "not have POSIX fork()");
 #endif
 }
 
@@ -146,9 +147,9 @@ pm_waitpid(pid_t         const pid,
     pid_t rc;
     rc = waitpid(pid, statusP, options);
     if (rc == (pid_t)-1) {
-        asprintfN(errorP, "Failed to wait for process exit.  "
-                  "waitpid() errno = %d (%s)",
-                  errno, strerror(errno));
+        pm_asprintf(errorP, "Failed to wait for process exit.  "
+                    "waitpid() errno = %d (%s)",
+                    errno, strerror(errno));
     } else {
         *exitedPidP = rc;
         *errorP = NULL;
@@ -172,7 +173,7 @@ pm_waitpidSimple(pid_t const pid) {
 
     if (error) {
         pm_errormsg("%s", error);
-        strfree(error);
+        pm_strfree(error);
         pm_longjmp();
     } else {
         assert(exitedPid != 0);
@@ -213,14 +214,14 @@ pm_message(const char format[], ...) {
 
     if (pm_showmessages) {
         const char * msg;
-        vasprintfN(&msg, format, args);
+        pm_vasprintf(&msg, format, args);
 
         if (userMessageFn)
             userMessageFn(msg);
         else
             fprintf(stderr, "%s: %s\n", pm_progname, msg);
 
-        strfree(msg);
+        pm_strfree(msg);
     }
     va_end(args);
 }
@@ -246,11 +247,11 @@ pm_errormsg(const char format[], ...) {
 
     va_start(args, format);
 
-    vasprintfN(&msg, format, args);
+    pm_vasprintf(&msg, format, args);
     
     errormsg(msg);
 
-    strfree(msg);
+    pm_strfree(msg);
 
     va_end(args);
 }
@@ -264,11 +265,11 @@ pm_error(const char format[], ...) {
 
     va_start(args, format);
 
-    vasprintfN(&msg, format, args);
+    pm_vasprintf(&msg, format, args);
     
     errormsg(msg);
 
-    strfree(msg);
+    pm_strfree(msg);
 
     va_end(args);
 
@@ -328,6 +329,10 @@ pm_allocarray(int const cols,
 
     pm_mallocarray2(&result, rows, cols, elementSize);
 
+    if (result == NULL)
+        pm_error("Failed to allocate a raster array of %u columns x %u rows",
+                 cols, rows);
+
     retval = result;
 
     return retval;
@@ -600,8 +605,9 @@ pm_proginit(int * const argcP, const char * argv[]) {
 
    This includes calling pm_init() to initialize the Netpbm libraries.
 -----------------------------------------------------------------------------*/
+    const char * const progname = pm_arg0toprogname(argv[0]);
+        /* points to static memory in this library */
     int argn, i;
-    const char * progname;
     bool showmessages;
     bool show_version;
         /* We're supposed to just show the version information, then exit the
@@ -611,13 +617,6 @@ pm_proginit(int * const argcP, const char * argv[]) {
         /* We're supposed to just tell user where to get help, then exit the
            program.
         */
-    
-    /* Extract program name. */
-    progname = strrchr( argv[0], '/');
-    if (progname == NULL)
-        progname = argv[0];
-    else
-        ++progname;
 
     pm_init(progname, 0);
 
@@ -684,6 +683,43 @@ pm_getMessage(void) {
 
 
 
+static void
+extractAfterLastSlash(const char * const fullPath,
+                      char *       const retval,
+                      size_t       const retvalSize) {
+    
+    char * slashPos;
+
+    /* Chop any directories off the left end */
+    slashPos = strrchr(fullPath, '/');
+
+    if (slashPos == NULL) {
+        strncpy(retval, fullPath, retvalSize);
+        retval[retvalSize-1] = '\0';
+    } else {
+        strncpy(retval, slashPos +1, retvalSize);
+        retval[retvalSize-1] = '\0';
+    }
+
+    /* Chop any .exe off the right end */
+    if (strlen(retval) >= 4 && strcmp(retval+strlen(retval)-4, ".exe") == 0)
+        retval[strlen(retval)-4] = 0;
+}
+
+
+
+#if MSVCRT
+static void
+splitpath(const char * const fullPath,
+          char *       const retval,
+          size_t       const retvalSize) {
+
+    _splitpath_s(fullPath, 0, 0,  0, 0,  retval, retvalSize,  0, 0);
+}
+#endif
+
+
+
 char *
 pm_arg0toprogname(const char arg0[]) {
 /*----------------------------------------------------------------------------
@@ -701,22 +737,11 @@ pm_arg0toprogname(const char arg0[]) {
    but truncated at 64 characters.
 -----------------------------------------------------------------------------*/
     static char retval[64+1];
-    char *slash_pos;
-
-    /* Chop any directories off the left end */
-    slash_pos = strrchr(arg0, '/');
-
-    if (slash_pos == NULL) {
-        strncpy(retval, arg0, sizeof(retval));
-        retval[sizeof(retval)-1] = '\0';
-    } else {
-        strncpy(retval, slash_pos +1, sizeof(retval));
-        retval[sizeof(retval)-1] = '\0';
-    }
-
-    /* Chop any .exe off the right end */
-    if (strlen(retval) >= 4 && strcmp(retval+strlen(retval)-4, ".exe") == 0)
-        retval[strlen(retval)-4] = 0;
+#if MSVCRT
+    splitpath(arg0, retval, sizeof(retval));
+#else
+    extractAfterLastSlash(arg0, retval, sizeof(retval));
+#endif
 
     return(retval);
 }
@@ -742,11 +767,11 @@ pm_parse_width(const char * const arg) {
     unsigned int width;
     const char * error;
 
-    interpret_uint(arg, &width, &error);
+    pm_interpret_uint(arg, &width, &error);
 
     if (error) {
         pm_error("'%s' is invalid as an image width.  %s", arg, error);
-        strfree(error);
+        pm_strfree(error);
     } else {
         if (width > INT_MAX-10)
             pm_error("Width %u is too large for computations.", width);
@@ -767,11 +792,11 @@ pm_parse_height(const char * const arg) {
     unsigned int height;
     const char * error;
 
-    interpret_uint(arg, &height, &error);
+    pm_interpret_uint(arg, &height, &error);
 
     if (error) {
         pm_error("'%s' is invalid as an image height.  %s", arg, error);
-        strfree(error);
+        pm_strfree(error);
     } else {
         if (height > INT_MAX-10)
             pm_error("Height %u is too large for computations.", height);
