@@ -29,6 +29,7 @@
     /* This makes the the x64() functions available on AIX */
 
 #include <unistd.h>
+#include <assert.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <stdarg.h>
@@ -111,6 +112,30 @@ tmpDir(void) {
 
 
 
+static int
+tempFileOpenFlags(void) {
+/*----------------------------------------------------------------------------
+  Open flags (argument to open()) suitable for a new temporary file  
+-----------------------------------------------------------------------------*/
+    int retval;
+
+    retval = 0
+        | O_CREAT
+        | O_RDWR
+#if !MSVCRT
+        | O_EXCL
+#endif
+#if MSVCRT
+        | O_BINARY
+#endif
+        ;
+
+    return retval;
+}
+
+
+
+
 static int
 mkstempx(char * const filenameBuffer) {
 /*----------------------------------------------------------------------------
@@ -145,9 +170,10 @@ mkstempx(char * const filenameBuffer) {
         else {
             int rc;
 
-            rc = open(filenameBuffer, O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
+            rc = open(filenameBuffer, tempFileOpenFlags(),
+                      PM_S_IWUSR | PM_S_IRUSR);
 
-            if (rc == 0) {
+            if (rc >= 0) {
                 fd = rc;
                 gotFile = TRUE;
             } else {
@@ -193,25 +219,25 @@ makeTmpfileWithTemplate(const char *  const filenameTemplate,
     filenameBuffer = strdup(filenameTemplate);
 
     if (filenameBuffer == NULL)
-        asprintfN(errorP, "Unable to allocate storage for temporary "
-                  "file name");
+        pm_asprintf(errorP, "Unable to allocate storage for temporary "
+                    "file name");
     else {
         int rc;
         
         rc = mkstemp2(filenameBuffer);
         
         if (rc < 0)
-            asprintfN(errorP,
-                      "Unable to create temporary file according to name "
-                      "pattern '%s'.  mkstemp() failed with errno %d (%s)",
-                      filenameTemplate, errno, strerror(errno));
+            pm_asprintf(errorP,
+                        "Unable to create temporary file according to name "
+                        "pattern '%s'.  mkstemp() failed with errno %d (%s)",
+                        filenameTemplate, errno, strerror(errno));
         else {
             *fdP = rc;
             *filenameP = filenameBuffer;
             *errorP = NULL;
         }
         if (*errorP)
-            strfree(filenameBuffer);
+            pm_strfree(filenameBuffer);
     }
 }
 
@@ -236,20 +262,20 @@ pm_make_tmpfile_fd(int *         const fdP,
     else
         dirseparator = "/";
     
-    asprintfN(&filenameTemplate, "%s%s%s%s", 
-              tmpdir, dirseparator, pm_progname, "_XXXXXX");
+    pm_asprintf(&filenameTemplate, "%s%s%s%s", 
+                tmpdir, dirseparator, pm_progname, "_XXXXXX");
 
-    if (filenameTemplate == NULL)
-        asprintfN(&error,
-                  "Unable to allocate storage for temporary file name");
+    if (filenameTemplate == pm_strsol)
+        pm_asprintf(&error,
+                    "Unable to allocate storage for temporary file name");
     else {
         makeTmpfileWithTemplate(filenameTemplate, fdP, filenameP, &error);
 
-        strfree(filenameTemplate);
+        pm_strfree(filenameTemplate);
     }
     if (error) {
         pm_errormsg("%s", error);
-        strfree(error);
+        pm_strfree(error);
         pm_longjmp();
     }
 }
@@ -269,7 +295,7 @@ pm_make_tmpfile(FILE **       const filePP,
     if (*filePP == NULL) {
         close(fd);
         unlink(*filenameP);
-        strfree(*filenameP);
+        pm_strfree(*filenameP);
 
         pm_error("Unable to create temporary file.  "
                  "fdopen() failed with errno %d (%s)",
@@ -289,7 +315,7 @@ pm_tmpfile(void) {
 
     unlink(tmpfile);
 
-    strfree(tmpfile);
+    pm_strfree(tmpfile);
 
     return fileP;
 }
@@ -306,7 +332,7 @@ pm_tmpfile_fd(void) {
 
     unlink(tmpfile);
 
-    strfree(tmpfile);
+    pm_strfree(tmpfile);
 
     return fd;
 }
@@ -523,6 +549,23 @@ pm_readbiglong(FILE * const ifP,
 
 
 
+int
+pm_readbiglong2(FILE *    const ifP,
+                int32_t * const lP) {
+    int rc;
+    long l;
+
+    rc = pm_readbiglong(ifP, &l);
+
+    assert((int32_t)l == l);
+
+    *lP = (int32_t)l;
+
+    return rc;
+}
+
+
+
 int
 pm_writebiglong(FILE * const ofP, 
                 long   const l) {
@@ -581,6 +624,23 @@ pm_readlittlelong(FILE * const ifP,
 
 
 
+int
+pm_readlittlelong2(FILE *    const ifP,
+                   int32_t * const lP) {
+    int rc;
+    long l;
+
+    rc = pm_readlittlelong(ifP, &l);
+
+    assert((int32_t)l == l);
+
+    *lP = (int32_t)l;
+
+    return rc;
+}
+
+
+
 int
 pm_writelittlelong(FILE * const ofP, 
                    long   const l) {
